@@ -614,8 +614,8 @@ nc_session_free(struct nc_session *session, void (*data_free)(void *))
     }
 
     /* stop notifications loop if any */
-    if ((session->side == NC_CLIENT) && session->opts.client.ntf_tid) {
-        session->opts.client.ntf_tid = NULL;
+    if ((session->side == NC_CLIENT) && ATOMIC_LOAD(session->opts.client.ntf_tid)) {
+        ATOMIC_STORE(session->opts.client.ntf_tid, (uintptr_t)NULL);
         /* the thread now knows it should quit */
     }
 
@@ -628,7 +628,7 @@ nc_session_free(struct nc_session *session, void (*data_free)(void *))
         } /* else failed to lock it, too bad */
     }
 
-    if ((session->side == NC_CLIENT) && (session->status == NC_STATUS_RUNNING)) {
+    if (session->side == NC_CLIENT) {
         /* cleanup message queues */
         /* notifications */
         for (contiter = session->opts.client.notifs; contiter; ) {
@@ -648,35 +648,37 @@ nc_session_free(struct nc_session *session, void (*data_free)(void *))
             free(p);
         }
 
-        /* send closing info to the other side */
-        ietfnc = ly_ctx_get_module(session->ctx, "ietf-netconf", NULL, 1);
-        if (!ietfnc) {
-            WRN("Session %u: missing ietf-netconf schema in context, unable to send <close-session>.", session->id);
-        } else {
-            close_rpc = lyd_new(NULL, ietfnc, "close-session");
-            nc_send_msg_io(session, NC_SESSION_FREE_LOCK_TIMEOUT, close_rpc);
-            lyd_free(close_rpc);
-            switch (nc_read_msg_poll_io(session, NC_CLOSE_REPLY_TIMEOUT, &rpl)) {
-            case NC_MSG_REPLY:
-                LY_TREE_FOR(rpl->child, child) {
-                    if (!strcmp(child->name, "ok") && child->ns && !strcmp(child->ns->value, NC_NS_BASE)) {
-                        break;
+        if (session->status == NC_STATUS_RUNNING) {
+            /* send closing info to the other side */
+            ietfnc = ly_ctx_get_module(session->ctx, "ietf-netconf", NULL, 1);
+            if (!ietfnc) {
+                WRN("Session %u: missing ietf-netconf schema in context, unable to send <close-session>.", session->id);
+            } else {
+                close_rpc = lyd_new(NULL, ietfnc, "close-session");
+                nc_send_msg_io(session, NC_SESSION_FREE_LOCK_TIMEOUT, close_rpc);
+                lyd_free(close_rpc);
+                switch (nc_read_msg_poll_io(session, NC_CLOSE_REPLY_TIMEOUT, &rpl)) {
+                case NC_MSG_REPLY:
+                    LY_TREE_FOR(rpl->child, child) {
+                        if (!strcmp(child->name, "ok") && child->ns && !strcmp(child->ns->value, NC_NS_BASE)) {
+                            break;
+                        }
                     }
+                    if (!child) {
+                        WRN("Session %u: the reply to <close-session> was not <ok> as expected.", session->id);
+                    }
+                    lyxml_free(session->ctx, rpl);
+                    break;
+                case NC_MSG_WOULDBLOCK:
+                    WRN("Session %u: timeout for receiving a reply to <close-session> elapsed.", session->id);
+                    break;
+                case NC_MSG_ERROR:
+                    ERR("Session %u: failed to receive a reply to <close-session>.", session->id);
+                    break;
+                default:
+                    /* cannot happen */
+                    break;
                 }
-                if (!child) {
-                    WRN("Session %u: the reply to <close-session> was not <ok> as expected.", session->id);
-                }
-                lyxml_free(session->ctx, rpl);
-                break;
-            case NC_MSG_WOULDBLOCK:
-                WRN("Session %u: timeout for receiving a reply to <close-session> elapsed.", session->id);
-                break;
-            case NC_MSG_ERROR:
-                ERR("Session %u: failed to receive a reply to <close-session>.", session->id);
-                break;
-            default:
-                /* cannot happen */
-                break;
             }
         }
 
@@ -796,14 +798,20 @@ nc_session_free(struct nc_session *session, void (*data_free)(void *))
             }
             /* change nc_sshcb_msg() argument, we need a RUNNING session and this one will be freed */
             if (session->flags & NC_SESSION_SSH_MSG_CB) {
-                for (siter = session->ti.libssh.next; siter->status != NC_STATUS_RUNNING; siter = siter->ti.libssh.next) {
+                siter = session->ti.libssh.next;
+                while (siter && siter->status != NC_STATUS_RUNNING) {
                     if (siter->ti.libssh.next == session) {
                         ERRINT;
                         break;
                     }
+                    siter = siter->ti.libssh.next;
                 }
+                /* siter may be NULL in case all the sessions terminated at the same time (socket was disconnected),
+                 * we set session to NULL because we do not expect any new message to arrive */
                 ssh_set_message_callback(session->ti.libssh.session, nc_sshcb_msg, siter);
-                siter->flags |= NC_SESSION_SSH_MSG_CB;
+                if (siter) {
+                    siter->flags |= NC_SESSION_SSH_MSG_CB;
+                }
             }
         }
 
@@ -1279,6 +1287,9 @@ nc_send_hello_io(struct nc_session *session)
         sid = NULL;
     } else {
         cpblts = nc_server_get_cpblts_version(session->ctx, LYS_VERSION_1);
+        if (!cpblts) {
+            return NC_MSG_ERROR;
+        }
 
         io_timeout = NC_SERVER_HELLO_TIMEOUT * 1000;
         sid = &session->id;
