@@ -1323,7 +1323,7 @@ _non_blocking_connect(int timeout, int *sock_pending, struct addrinfo *res, stru
         ERR("getsockopt() failed (%s).", strerror(errno));
         goto cleanup;
     }
-    if (error == ECONNREFUSED) {
+    if (error) {
         /* network connection failed, try another resource */
         VRB("getsockopt() error (%s).", strerror(error));
         errno = error;
@@ -1380,9 +1380,14 @@ nc_sock_connect(const char *host, uint16_t port, int timeout, struct nc_keepaliv
 
         for (res = res_list; res != NULL; res = res->ai_next) {
             sock = _non_blocking_connect(timeout, sock_pending, res, ka);
-            if (sock == -1 && (!sock_pending || *sock_pending == -1)) {
-                /* try the next resource */
-                continue;
+            if (sock == -1) {
+                if (!sock_pending || *sock_pending == -1) {
+                    /* try the next resource */
+                    continue;
+                } else {
+                    /* timeout, keep pending socket */
+                    return -1;
+                }
             }
             VRB("Successfully connected to %s:%s over %s.", host, port_s, (res->ai_family == AF_INET6) ? "IPv6" : "IPv4");
 
@@ -1488,7 +1493,7 @@ get_msg(struct nc_session *session, int timeout, uint64_t msgid, struct lyxml_el
 
     /* we read notif, want a rpc-reply */
     if (msgid && (msgtype == NC_MSG_NOTIF)) {
-        if (!session->opts.client.ntf_tid) {
+        if (!ATOMIC_LOAD(session->opts.client.ntf_tid)) {
             ERR("Session %u: received a <notification> but session is not subscribed.", session->id);
             lyxml_free(session->ctx, xml);
             return NC_MSG_ERROR;
@@ -2054,7 +2059,7 @@ nc_session_ntf_thread_running(const struct nc_session *session)
         return 0;
     }
 
-    return session->opts.client.ntf_tid ? 1 : 0;
+    return ATOMIC_LOAD(session->opts.client.ntf_tid) ? 1 : 0;
 }
 
 API void
@@ -2169,7 +2174,7 @@ nc_recv_notif(struct nc_session *session, int timeout, struct nc_notif **notif)
         }
 
         /* notification body */
-        (*notif)->tree = lyd_parse_xml(session->ctx, &xml->child, LYD_OPT_NOTIF | LYD_OPT_DESTRUCT | LYD_OPT_NOEXTDEPS
+        (*notif)->tree = lyd_parse_xml(session->ctx, &xml->child, LYD_OPT_NOTIF | LYD_OPT_DESTRUCT | LYD_OPT_TRUSTED
                                        | (session->flags & NC_SESSION_CLIENT_NOT_STRICT ? 0 : LYD_OPT_STRICT), NULL);
         lyxml_free(session->ctx, xml);
         xml = NULL;
@@ -2209,9 +2214,9 @@ nc_recv_notif_thread(void *arg)
     free(ntarg);
 
     /* remember our allocated tid, we will be freeing it */
-    ntf_tid = (pthread_t *)session->opts.client.ntf_tid;
+    ntf_tid = (pthread_t *)ATOMIC_LOAD(session->opts.client.ntf_tid);
 
-    while (session->opts.client.ntf_tid) {
+    while (ATOMIC_LOAD(session->opts.client.ntf_tid)) {
         msgtype = nc_recv_notif(session, NC_CLIENT_NOTIF_THREAD_SLEEP / 1000, &notif);
         if (msgtype == NC_MSG_NOTIF) {
             notif_clb(session, notif);
@@ -2230,7 +2235,7 @@ nc_recv_notif_thread(void *arg)
     }
 
     VRB("Session %u: notification thread exit.", session->id);
-    session->opts.client.ntf_tid = NULL;
+    ATOMIC_STORE(session->opts.client.ntf_tid, (uintptr_t)NULL);
     free(ntf_tid);
     return NULL;
 }
@@ -2239,6 +2244,7 @@ API int
 nc_recv_notif_dispatch(struct nc_session *session, void (*notif_clb)(struct nc_session *session, const struct nc_notif *notif))
 {
     struct nc_ntf_thread_arg *ntarg;
+    pthread_t *tid;
     int ret;
 
     if (!session) {
@@ -2250,7 +2256,7 @@ nc_recv_notif_dispatch(struct nc_session *session, void (*notif_clb)(struct nc_s
     } else if ((session->status != NC_STATUS_RUNNING) || (session->side != NC_CLIENT)) {
         ERR("Session %u: invalid session to receive Notifications.", session->id);
         return -1;
-    } else if (session->opts.client.ntf_tid) {
+    } else if (ATOMIC_LOAD(session->opts.client.ntf_tid)) {
         ERR("Session %u: separate notification thread is already running.", session->id);
         return -1;
     }
@@ -2263,20 +2269,21 @@ nc_recv_notif_dispatch(struct nc_session *session, void (*notif_clb)(struct nc_s
     ntarg->session = session;
     ntarg->notif_clb = notif_clb;
 
-    /* just so that nc_recv_notif_thread() does not immediately exit, the value does not matter */
-    session->opts.client.ntf_tid = malloc(sizeof *session->opts.client.ntf_tid);
-    if (!session->opts.client.ntf_tid) {
+    tid = malloc(sizeof *tid);
+    if (!tid) {
         ERRMEM;
         free(ntarg);
         return -1;
     }
+    /* just so that nc_recv_notif_thread() does not immediately exit, the value does not matter */
+    ATOMIC_STORE(session->opts.client.ntf_tid, (uintptr_t)tid);
 
-    ret = pthread_create((pthread_t *)session->opts.client.ntf_tid, NULL, nc_recv_notif_thread, ntarg);
+    ret = pthread_create(tid, NULL, nc_recv_notif_thread, ntarg);
     if (ret) {
         ERR("Session %u: failed to create a new thread (%s).", strerror(errno));
         free(ntarg);
-        free((pthread_t *)session->opts.client.ntf_tid);
-        session->opts.client.ntf_tid = NULL;
+        free(tid);
+        ATOMIC_STORE(session->opts.client.ntf_tid, (uintptr_t)NULL);
         return -1;
     }
 
@@ -2647,7 +2654,11 @@ nc_send_rpc(struct nc_session *session, struct nc_rpc *rpc, int timeout, uint64_
 
         data = lyd_new(NULL, mod, "kill-session");
         sprintf(str, "%u", rpc_k->sid);
-        lyd_new_leaf(data, mod, "session-id", str);
+        node = lyd_new_leaf(data, mod, "session-id", str);
+        if (!node) {
+            lyd_free(data);
+            return NC_MSG_ERROR;
+        }
         break;
 
     case NC_RPC_COMMIT:
@@ -2655,12 +2666,20 @@ nc_send_rpc(struct nc_session *session, struct nc_rpc *rpc, int timeout, uint64_
 
         data = lyd_new(NULL, mod, "commit");
         if (rpc_com->confirmed) {
-            lyd_new_leaf(data, mod, "confirmed", NULL);
+            node = lyd_new_leaf(data, mod, "confirmed", NULL);
+            if (!node) {
+                lyd_free(data);
+                return NC_MSG_ERROR;
+            }
         }
 
         if (rpc_com->confirm_timeout) {
             sprintf(str, "%u", rpc_com->confirm_timeout);
-            lyd_new_leaf(data, mod, "confirm-timeout", str);
+            node = lyd_new_leaf(data, mod, "confirm-timeout", str);
+            if (!node) {
+                lyd_free(data);
+                return NC_MSG_ERROR;
+            }
         }
 
         if (rpc_com->persist) {
