@@ -773,7 +773,7 @@ nc_accept_inout(int fdin, int fdout, const char *username, struct nc_session **s
     (*session)->ctx = server_opts.ctx;
 
     /* assign new SID atomically */
-    (*session)->id = ATOMIC_INC(&server_opts.new_session_id);
+    (*session)->id = ATOMIC_INC(server_opts.new_session_id);
 
     /* NETCONF handshake */
     msgtype = nc_handshake_io(*session);
@@ -1716,8 +1716,6 @@ nc_ps_clear(struct nc_pollsession *ps, int all, void (*data_free)(void *))
     nc_ps_unlock(ps, q_id, __func__);
 }
 
-#if defined(NC_ENABLED_SSH) || defined(NC_ENABLED_TLS)
-
 static int
 nc_get_uid(int sock, uid_t *uid)
 {
@@ -2103,13 +2101,24 @@ nc_server_endpt_set_address_port(const char *endpt_name, const char *address, ui
     }
 
     if (sock > -1) {
-#if defined(NC_ENABLED_SSH) && defined(NC_ENABLED_TLS)
-        VRB("Listening on %s:%u for %s connections.", address, port, (endpt->ti == NC_TI_LIBSSH ? "SSH" : "TLS"));
-#elif defined(NC_ENABLED_SSH)
-        VRB("Listening on %s:%u for SSH connections.", address, port);
-#else
-        VRB("Listening on %s:%u for TLS connections.", address, port);
+        switch (endpt->ti) {
+        case NC_TI_UNIX:
+            VRB("Listening on %s for UNIX connections.", address);
+            break;
+#ifdef NC_ENABLED_SSH
+        case NC_TI_LIBSSH:
+            VRB("Listening on %s:%u for SSH connections.", address, port);
+            break;
 #endif
+#ifdef NC_ENABLED_TLS
+        case NC_TI_OPENSSL:
+            VRB("Listening on %s:%u for TLS connections.", address, port);
+            break;
+#endif
+        default:
+            ERRINT;
+            break;
+        }
     }
 
 cleanup:
@@ -2128,12 +2137,16 @@ nc_server_endpt_set_address(const char *endpt_name, const char *address)
     return nc_server_endpt_set_address_port(endpt_name, address, 0);
 }
 
+#if defined(NC_ENABLED_SSH) || defined(NC_ENABLED_TLS)
+
 API int
 nc_server_endpt_set_port(const char *endpt_name, uint16_t port)
 {
     return nc_server_endpt_set_address_port(endpt_name, NULL, port);
 }
 
+#endif
+
 API int
 nc_server_endpt_set_perms(const char *endpt_name, mode_t mode, uid_t uid, gid_t gid)
 {
@@ -2336,7 +2349,7 @@ nc_accept(int timeout, struct nc_session **session)
     pthread_rwlock_unlock(&server_opts.endpt_lock);
 
     /* assign new SID atomically */
-    (*session)->id = ATOMIC_INC(&server_opts.new_session_id);
+    (*session)->id = ATOMIC_INC(server_opts.new_session_id);
 
     /* NETCONF handshake */
     msgtype = nc_handshake_io(*session);
@@ -2363,6 +2376,8 @@ cleanup:
     return msgtype;
 }
 
+#if defined(NC_ENABLED_SSH) || defined(NC_ENABLED_TLS)
+
 /* client is expected to be locked */
 static int
 _nc_server_ch_client_del_endpt(struct nc_ch_client *client, const char *endpt_name, NC_TRANSPORT_IMPL ti)
@@ -2482,7 +2497,7 @@ nc_server_ch_add_client(const char *name)
     client = &server_opts.ch_clients[server_opts.ch_client_count - 1];
 
     client->name = lydict_insert(server_opts.ctx, name, 0);
-    client->id = ATOMIC_INC(&server_opts.new_client_id);
+    client->id = ATOMIC_INC(server_opts.new_client_id);
     client->ch_endpts = NULL;
     client->ch_endpt_count = 0;
     client->conn_type = 0;
@@ -3119,7 +3134,7 @@ nc_connect_ch_endpt(struct nc_ch_endpt *endpt, struct nc_session **session)
     }
 
     /* assign new SID atomically */
-    (*session)->id = ATOMIC_INC(&server_opts.new_session_id);
+    (*session)->id = ATOMIC_INC(server_opts.new_session_id);
 
     /* NETCONF handshake */
     msgtype = nc_handshake_io(*session);
